@@ -1,44 +1,72 @@
 import functools
-from .engin import Engin
 from . import sql_support
-from .constant import MYSQL_CONNECTOR
+from .engine import Engine
 from .init_import import import_driver
 from .log_support import logger, insert_log, save_log, get_log
 from .support import DBCtx, ConnectionCtx, Dict, MultiColumnsError, TransactionCtx, try_commit, DBError, DB_LOCK
+from .constant import MYSQL_CONNECTOR, SQLITE, PARAM_DRIVER, PARAM_SHOW_SQL, PARAM_TRANS_PLACEHOLDER, PARAM_DEBUG, PARAM_POOL_SIZE
 
 _DB_CTX = None
-_SHOW_SQL = False
 
 
-def init_db(driver='', pool_size=0, show_sql=False, debug=False, **kwargs):
-    global _DB_CTX, _SHOW_SQL
-    if debug:
+def init_db(*args, **kwargs):
+    """
+    Compliant with the Python DB API 2.0 (PEP-249).
+
+    from sqlexec
+    sqlexec.init_db('test.db', driver='sqlite3', show_sql=True, debug=True)
+    or
+    sqlexec.init_db("postgres://user:password@127.0.0.1:5432/testdb", driver='psycopg2', pool_size=5, show_sql=True, debug=True)
+    or
+    sqlexec.init_db(user='root', password='xxx', host='127.0.0.1', port=3306, database='testdb', driver='pymysql', pool_size=5, show_sql=True, debug=True)
+
+    Addition parameters:
+    :param driver=None: str, import driver, 'import pymysql'
+    :param pool_size=0: int, default 0, size of connection pool
+    :param show_sql=False: bool,  if True, print sql
+    :param debug=False: bool, if True, print debug context
+    :param trans_placeholder=True: bool, if True, sql placeholder '?' --> '%s'
+
+    Other parameters of connection pool refer to DBUtils: https://webwareforpython.github.io/DBUtils/main.html#pooleddb-pooled-db
+    """
+
+    global _DB_CTX
+    pool_size = 0
+    driver = kwargs.pop(PARAM_DRIVER) if 'PARAM_DRIVER' in kwargs else None
+    show_sql = kwargs.pop(PARAM_SHOW_SQL) if PARAM_SHOW_SQL in kwargs else False
+    trans_placeholder = kwargs.pop(PARAM_TRANS_PLACEHOLDER) if PARAM_TRANS_PLACEHOLDER in kwargs else True
+    engine, driver, creator = import_driver(driver)
+    prepared = MYSQL_CONNECTOR == driver
+    if PARAM_DEBUG in kwargs and kwargs.pop(PARAM_DEBUG):
         from logging import DEBUG
         logger.setLevel(DEBUG)
 
-    prepared = MYSQL_CONNECTOR == driver
-    engin, driver, creator = import_driver(driver)
-    if pool_size <= 0:
-        connect = lambda: creator.connect(**kwargs)
-    elif prepared:
-        # mysql.connector 用自带连接池
-        kwargs['pool_size'] = pool_size
-        connect = lambda: creator.connect(**kwargs)
-    else:
+    if PARAM_POOL_SIZE in kwargs:
+        if prepared:
+            # mysql.connector 用自带连接池
+            pool_size = kwargs[PARAM_POOL_SIZE]
+        else:
+            pool_size = kwargs.pop(PARAM_POOL_SIZE)
+
+    pool_args = ['mincached', 'maxcached', 'maxshared', 'maxconnections', 'blocking', 'maxusage', 'setsession', 'reset', 'failures', 'ping']
+    pool_kwargs = {key: kwargs.pop(key) for key in pool_args if key in kwargs}
+    connect = lambda: creator.connect(*args, **kwargs)
+    if pool_size >= 1 and not prepared:
         from .pooling import pooled_connect
-        connect = pooled_connect(creator, pool_size, **kwargs)
+        connect = pooled_connect(connect, pool_size, **pool_kwargs)
 
     with DB_LOCK:
         if _DB_CTX is not None:
             raise DBError('DB is already initialized.')
-        _SHOW_SQL = show_sql
         _DB_CTX = DBCtx(connect=connect, prepared=prepared)
 
-    Engin.init(engin)
+    if SQLITE == engine:
+        trans_placeholder = False
+    Engine.init0(engine, trans_placeholder, show_sql)
     if pool_size > 0:
-        logger.info("Inited db engin <%s> of %s with driver: '%s' and pool size: %d." % (hex(id(_DB_CTX)), engin, driver, pool_size))
+        logger.info("Inited db engine <%s> of %s with driver: '%s' and pool size: %d." % (hex(id(_DB_CTX)), engine, driver, pool_size))
     else:
-        logger.info("Inited db engin <%s> of %s with driver: '%s'." % (hex(id(_DB_CTX)), engin, driver))
+        logger.info("Inited db engine <%s> of %s with driver: '%s'." % (hex(id(_DB_CTX)), engine, driver))
 
 
 def connection():
@@ -106,7 +134,7 @@ def execute(sql: str, *args):
     """
     global _DB_CTX
     cursor = None
-    sql = _before_execute('execute', sql.strip(), *args)
+    sql = Engine.before_execute_intf('execute', sql.strip(), *args)
     try:
         cursor = _DB_CTX.connection.cursor()
         cursor.execute(sql, args)
@@ -155,7 +183,7 @@ def save_sql(select_key: str, sql: str, *args):
     global _DB_CTX
     cursor = None
     logger.debug("Exec func 'sqlexec.%s', 'select_key': %s \n\t sql: %s \n\t args: %s" % ('save_sql', select_key, sql, args))
-    sql = _before_execute('save_sql', sql, *args)
+    sql = Engine.before_execute_intf('save_sql', sql, *args)
     try:
         cursor = _DB_CTX.connection.cursor()
         cursor.execute(sql, args)
@@ -193,7 +221,7 @@ def batch_execute(sql: str, *args):
     cursor = None
     if isinstance(args[0], dict):
         sql, args = sql_support.batch_named_sql_args(sql, *args)
-    sql = _before_execute('batch_execute', sql.strip(), *args)
+    sql = Engine.before_execute_intf('batch_execute', sql.strip(), *args)
     args = sql_support.get_batch_args(*args)
     try:
         cursor = _DB_CTX.cursor()
@@ -231,7 +259,7 @@ def select(sql: str, *args):
     """
     global _DB_CTX
     cursor = None
-    sql = _before_execute('select', sql.strip(), *args)
+    sql = Engine.before_execute_intf('select', sql.strip(), *args)
     try:
         cursor = _DB_CTX.cursor()
         cursor.execute(sql, args)
@@ -249,7 +277,7 @@ def select_one(sql: str, *args):
     """
     global _DB_CTX
     cursor = None
-    sql = _before_execute('select_one', sql.strip(), *args)
+    sql = Engine.before_execute_intf('select_one', sql.strip(), *args)
     try:
         cursor = _DB_CTX.cursor()
         cursor.execute(sql, args)
@@ -273,7 +301,7 @@ def query(sql: str, *args):
     """
     global _DB_CTX
     cursor = None
-    sql = _before_execute('query', sql.strip(), *args)
+    sql = Engine.before_execute_intf('query', sql.strip(), *args)
     try:
         cursor = _DB_CTX.cursor()
         cursor.execute(sql, args)
@@ -296,7 +324,7 @@ def query_one(sql: str, *args):
     """
     global _DB_CTX
     cursor = None
-    sql = _before_execute('query_one', sql.strip(), *args)
+    sql = Engine.before_execute_intf('query_one', sql.strip(), *args)
     try:
         cursor = _DB_CTX.cursor()
         cursor.execute(sql, args)
@@ -320,11 +348,3 @@ def get_connection():
     global _DB_CTX
     _DB_CTX.try_init()
     return _DB_CTX.connection
-
-
-def _before_execute(function: str, sql: str, *args):
-    if _SHOW_SQL:
-        logger.info("Exec func 'sqlexec.%s' \n\tSQL: %s \n\tARGS: %s" % (function, sql, args))
-    if '%' in sql and 'like' in sql.lower():
-        sql = sql.replace('%', '%%').replace('%%%%', '%%')
-    return sql.replace('?', '%s')
